@@ -8,6 +8,7 @@ import json
 import os
 import re
 import time
+import shutil
 
 try:
     from urlparse import urlparse
@@ -17,6 +18,7 @@ except ImportError:
 
 from kazoo.client import KazooClient
 from kazoo.exceptions import (
+    NoAuthError,
     NodeExistsError,
     NoNodeError,
     NoChildrenForEphemeralsError,
@@ -52,7 +54,21 @@ def url_join(url_root, child_path):
 
 class CopyError(Exception):
     """ base exception for Copy errors """
-    pass
+
+    def __init__(self, message, early_error=False):
+        super(CopyError, self).__init__(message)
+        self._early_error = early_error
+
+    @property
+    def is_early_error(self):
+        return self._early_error
+
+
+class AuthError(CopyError):
+    """ authentication exception for Copy """
+    def __init__(self, operation, path):
+        super(AuthError, self).__init__(
+            "Permission denied: Could not %s znode %s." % (operation, path))
 
 
 class PathValue(object):
@@ -167,10 +183,22 @@ class Proxy(ProxyType("ProxyBase", (object,), {})):
     def children_of(self):
         raise NotImplementedError("children_of must be implemented")
 
-    def copy(self, dst, recursive, max_items):
+    def delete_path_recursively(self):
+        raise NotImplementedError("delete_path must be implemented")
+
+    def copy(self, dst, recursive, max_items, mirror):
+        opname = "Copy" if not mirror else "Mirror"
+
         # basic sanity check
+        if mirror and self.scheme == "zk" and dst.scheme == "file":
+            raise CopyError("Mirror from zk to fs isn't supported", True)
+
         if recursive and self.scheme == "zk" and dst.scheme == "file":
-            raise CopyError("Recursive copy from zk to fs isn't supported")
+            raise CopyError("Recursive %s from zk to fs isn't supported" %
+                            opname.lower(), True)
+
+        if mirror and not recursive:
+            raise CopyError("Mirroring must be recursive", True)
 
         start = time.time()
 
@@ -179,29 +207,40 @@ class Proxy(ProxyType("ProxyBase", (object,), {})):
 
         with self:
             with dst:
-                self.do_copy(dst)
+                if mirror:
+                    dst_children = set(c for c in dst.children_of())
+
+                self.do_copy(dst, opname)
+
                 if recursive:
                     for i, child in enumerate(self.children_of()):
+                        if mirror and child in dst_children:
+                            dst_children.remove(child)
                         if max_items > 0 and i == max_items:
                             break
                         self.set_url(url_join(src_url, child))
                         dst.set_url(url_join(dst_url, child))
-                        self.do_copy(dst)
+                        self.do_copy(dst, opname)
 
                         # reset to base urls
                         self.set_url(src_url)
                         dst.set_url(dst_url)
 
+                if mirror:
+                    for child in dst_children:
+                        dst.set_url(url_join(dst_url, child))
+                        dst.delete_path_recursively()
+
         end = time.time()
 
-        print("Copying took %.2f secs" % (round(end - start, 2)))
+        print("%sing took %.2f secs" % (opname, round(end - start, 2)))
 
-    def do_copy(self, dst):
+    def do_copy(self, dst, opname):
         if self.verbose:
             if self.async:
-                print("Copying (asynchronously) from %s to %s" % (self.url, dst.url))
+                print("%sing (asynchronously) from %s to %s" % (opname, self.url, dst.url))
             else:
-                print("Copying from %s to %s" % (self.url, dst.url))
+                print("%sing from %s to %s" % (opname, self.url, dst.url))
 
         dst.write_path(self.read_path())
 
@@ -250,7 +289,11 @@ class ZKProxy(Proxy):
         self.disconnect()
 
     def check_path(self):
-        retval = True if self.client.exists(self.path) else False
+        try:
+            retval = True if self.client.exists(self.path) else False
+        except NoAuthError:
+            raise AuthError("read", self.path)
+
         if retval is not self.exists:
             if self.exists:
                 error = "znode %s in %s doesn't exist" % \
@@ -260,10 +303,13 @@ class ZKProxy(Proxy):
             raise CopyError(error)
 
     def read_path(self):
-        # TODO: propose a new ZK opcode (GetWithACLs) so we can do this in 1 rt
-        value = self.get_value(self.path)
-        acl, _ = self.client.get_acls(self.path)
-        return self.ZKPathValue(value, acl)
+        try:
+            # TODO: propose a new ZK opcode (GetWithACLs) so we can do this in 1 rt
+            value = self.get_value(self.path)
+            acl, _ = self.client.get_acls(self.path)
+            return self.ZKPathValue(value, acl)
+        except NoAuthError:
+            raise AuthError("read", self.path)
 
     def write_path(self, path_value):
         if isinstance(path_value, self.ZKPathValue):
@@ -272,15 +318,20 @@ class ZKProxy(Proxy):
             acl = [ACLReader.from_dict(a) for a in path_value.acl]
 
         if self.client.exists(self.path):
-            value = self.get_value(self.path)
-            if path_value.value != value:
-                self.client.set(self.path, path_value.value)
+            try:
+                value = self.get_value(self.path)
+                if path_value.value != value:
+                    self.client.set(self.path, path_value.value)
+            except NoAuthError:
+                raise AuthError("write", self.path)
         else:
             try:
                 # Kazoo's create() doesn't handle acl=[] correctly
                 # See: https://github.com/python-zk/kazoo/pull/164
                 acl = acl or None
                 self.client.create(self.path, path_value.value, acl=acl, makepath=True)
+            except NoAuthError:
+                raise AuthError("create", self.path)
             except NodeExistsError:
                 raise CopyError("Node %s exists" % (self.path))
             except NoNodeError:
@@ -291,13 +342,26 @@ class ZKProxy(Proxy):
                 raise CopyError("ZooKeeper server error")
 
     def get_value(self, path):
-        if hasattr(self.client, 'get_bytes'):
-            v, _ = self.client.get_bytes(path)
-        else:
-            v, _ = self.client.get(path)
+        try:
+            if hasattr(self.client, 'get_bytes'):
+                v, _ = self.client.get_bytes(path)
+            else:
+                v, _ = self.client.get(path)
+        except NoAuthError:
+            raise AuthError("read", path)
 
         return v
 
+    def delete_path_recursively(self):
+        try:
+            self.client.delete(self.path, recursive=True)
+        except NoNodeError:
+            pass
+        except NoAuthError:
+            raise AuthError("delete", self.path)
+        except ZookeeperError:
+            raise CopyError("Zookeeper server error")
+
     def children_of(self):
         if self.async:
             return AsyncWalker(self.client).walk(self.path.rstrip("/"))
@@ -310,16 +374,27 @@ class ZKProxy(Proxy):
         """
         full_path = "%s/%s" % (root_path, branch_path) if branch_path else root_path
 
-        for child in self.client.get_children(full_path):
+        try:
+            children = self.client.get_children(full_path)
+        except NoNodeError:
+            children = set()
+        except NoAuthError:
+            raise AuthError("read children", full_path)
+
+        for child in children:
             child_path = "%s/%s" % (branch_path, child) if branch_path else child
-            stat = self.client.exists("%s/%s" % (root_path, child_path))
+
+            try:
+                stat = self.client.exists("%s/%s" % (root_path, child_path))
+            except NoAuthError:
+                raise AuthError("read", child)
+
             if stat is None or stat.ephemeralOwner != 0:
                 continue
             yield child_path
             for new_path in self.zk_walk(root_path, child_path):
                 yield new_path
 
-
 class FileProxy(Proxy):
     SCHEME = "file"
 
@@ -346,7 +421,6 @@ class FileProxy(Proxy):
 
     def write_path(self, path_value):
         """ this will overwrite dst path - be careful """
-
         parent_dir = os.path.dirname(self.path)
         try:
             os.makedirs(parent_dir)
@@ -366,6 +440,9 @@ class FileProxy(Proxy):
             for filename in files:
                 yield "%s/%s" % (path, filename) if path != "" else filename
 
+    def delete_path_recursively(self):
+        shutil.rmtree(self.path, True)
+
 
 class JSONProxy(Proxy):
     """ read/write from JSON files discovered via:
@@ -458,3 +535,10 @@ class JSONProxy(Proxy):
         for child in self._tree.keys():
             if good(child):
                 yield child[offs:]
+
+    def delete_path_recursively(self):
+        if self.path in self._tree:
+            # build a set from the iterable so we don't change the dictionary during iteration
+            for c in set(self.children_of()):
+                self._tree.pop(url_join(self.path, c))
+            self._tree.pop(self.path)
