@@ -27,6 +27,7 @@ It supports the basic ops plus a few handy extensions:
 
 from __future__ import print_function
 
+from contextlib import contextmanager
 from functools import partial, wraps
 import json
 import os
@@ -63,9 +64,19 @@ from .augumented_cmd import (
     Required,
 )
 from .copy import CopyError, Proxy
+from .keys import Keys
 from .watcher import get_child_watcher
 from .watch_manager import get_watch_manager
-from .util import Netloc, pretty_bytes, to_bool, to_int, decoded, prompt_yes_no
+from .util import (
+    Netloc,
+    pretty_bytes,
+    to_bool,
+    to_float,
+    to_int,
+    decoded,
+    prompt_yes_no,
+    invalid_hosts
+)
 
 
 def connected(func):
@@ -145,6 +156,7 @@ class Shell(AugumentedCmd):
         self._async = async
         self._zk = None
         self.connected = False
+        self.state_transitions_enabled = True
 
         if len(self._hosts) > 0:
             self._connect(self._hosts)
@@ -739,48 +751,60 @@ child_watches=%s"""
             if params.match == "" or params.match in hcmd:
                 self.do_output("%s", hcmd)
 
-    @ensure_params(Optional("host"))
+    @ensure_params(Optional("hosts"))
     def do_mntr(self, params):
         """
-        runs the mntr 4 letter command on current or given host
+        runs the mntr 4 letter command on current or given hosts
         """
-        host = params.host if params.host != "" else None
+        hosts = params.hosts if params.hosts != "" else None
+
+        if hosts is not None and invalid_hosts(hosts):
+            self.do_output("List of hosts has the wrong syntax.")
+            return
 
         if self._zk is None:
             self._zk = AugumentedClient()
 
         try:
-            self.do_output(self._zk.mntr(host))
+            self.do_output(self._zk.mntr(hosts))
         except AugumentedClient.CmdFailed as ex:
             self.do_output(str(ex))
 
-    @ensure_params(Optional("host"))
+    @ensure_params(Optional("hosts"))
     def do_cons(self, params):
         """
-        runs the cons 4 letter command on current or given host
+        runs the cons 4 letter command on current or given hosts
         """
-        host = params.host if params.host != "" else None
+        hosts = params.hosts if params.hosts != "" else None
+
+        if hosts is not None and invalid_hosts(hosts):
+            self.do_output("List of hosts has the wrong syntax.")
+            return
 
         if self._zk is None:
             self._zk = AugumentedClient()
 
         try:
-            self.do_output(self._zk.cons(host))
+            self.do_output(self._zk.cons(hosts))
         except AugumentedClient.CmdFailed as ex:
             self.do_output(str(ex))
 
-    @ensure_params(Optional("host"))
+    @ensure_params(Optional("hosts"))
     def do_dump(self, params):
         """
-        runs the cons 4 letter command on current or given host
+        runs the cons 4 letter command on current or given hosts
         """
-        host = params.host if params.host != "" else None
+        hosts = params.hosts if params.hosts != "" else None
+
+        if hosts is not None and invalid_hosts(hosts):
+            self.do_output("List of hosts has the wrong syntax.")
+            return
 
         if self._zk is None:
             self._zk = AugumentedClient()
 
         try:
-            self.do_output(self._zk.dump(host))
+            self.do_output(self._zk.dump(hosts))
         except AugumentedClient.CmdFailed as ex:
             self.do_output(str(ex))
 
@@ -970,32 +994,23 @@ child_watches=%s"""
 
         Or recursively:
 
-        json_get /configs endpoint.clientPorts true
+        json_get /configs endpoint.clientPort true
         primary_service: 32768
         secondary_service: 32769
+
+        You can also use template strings to access various keys at once:
+
+        json_get /configs/primary_service '#{endpoint.ipAddress}:#{endpoint.clientPort}'
+        10.2.2.3:32768
         """
-        if re.match(r"\w+(?:\.\w+)*$", params.keys) is None:
-            self.do_output("Bad key syntax, should be: key1.key2...")
+        try:
+            Keys.validate(params.keys)
+        except Keys.Bad as ex:
+            self.do_output(str(ex))
             return
 
-        class MissingKey(Exception): pass
         class BadJSON(Exception): pass
 
-        def get_from_obj(obj, keys):
-            current = obj
-            for key in keys.split("."):
-                if type(current) == list:
-                    try:
-                        key = int(key)
-                    except TypeError:
-                        raise MissingKey(key)
-
-                try:
-                    current = current[key]
-                except (IndexError, KeyError, TypeError) as ex:
-                    raise MissingKey(key)
-            return current
-
         def get(path, keys, print_path):
             jstr, _ = self._zk.get(path)
 
@@ -1007,7 +1022,7 @@ child_watches=%s"""
             except ValueError:
                 raise BadJSON(path)
 
-            value = get_from_obj(obj, keys)
+            value = Keys.value(obj, keys)
 
             if print_path:
                 self.do_output("%s: %s", os.path.basename(path), value)
@@ -1026,11 +1041,62 @@ child_watches=%s"""
                 get(cpath, params.keys, print_path)
             except BadJSON as ex:
                 self.do_output("Path %s has bad JSON.", cpath)
-            except MissingKey as ex:
+            except Keys.Missing as ex:
                 self.do_output("Path %s is missing key %s.", cpath, ex)
 
     complete_json_get = _complete_path
 
+    @contextmanager
+    def transitions_disabled(self):
+        """
+        use this when you want to ignore state transitions (i.e.: inside loop)
+        """
+        self.state_transitions_enabled = False
+        try:
+            yield
+        except KeyboardInterrupt:
+            pass
+        self.state_transitions_enabled = True
+
+    @connected
+    @ensure_params(Required("repeat"), Required("pause"), Multi("cmds"))
+    def do_loop(self, params):
+        """
+        runs <cmds> <repeat> times, with a pause of <pause> secs inbetween
+
+        example:
+
+        loop 3 0 "get /foo"
+
+        Or multiple cmds:
+
+        loop 3 0 "get /foo" "get /bar"
+
+        """
+        repeat = to_int(params.repeat, -1)
+        if repeat < 0:
+            self.do_output("<repeat> must be >= 0.")
+            return
+        pause = to_float(params.pause, -1)
+        if pause < 0:
+            self.do_output("<pause> must be >= 0.")
+            return
+
+        cmds = params.cmds
+        i = 0
+        with self.transitions_disabled():
+            while True:
+                for cmd in cmds:
+                    try:
+                        self.onecmd(cmd)
+                    except Exception as ex:
+                        self.do_output("Command failed: %s.", ex)
+                if pause > 0.0:
+                    time.sleep(pause)
+                i += 1
+                if repeat > 0 and i >= repeat:
+                    break
+
     @ensure_params(Required("hosts"))
     def do_connect(self, params):
         """
