@@ -1,6 +1,7 @@
 """ helpers """
 
 from collections import namedtuple
+import re
 import sys
 
 from distutils.util import strtobool
@@ -46,6 +47,15 @@ def to_int(sint, default):
     except ValueError:
         return default
 
+
+def to_float(sfloat, default):
+    """ get a float from an str """
+    try:
+        return float(sfloat)
+    except ValueError:
+        return default
+
+
 def decoded(s):
     if PYTHON3:
         return str.encode(s).decode('unicode_escape')
@@ -79,3 +89,11 @@ class Netloc(namedtuple("Netloc", "host scheme credential")):
             scheme, credential = scheme_credential.split(":", 1)
 
         return cls(host, scheme, credential)
+
+
+_hosts = re.compile(r"\A\w+(?:\.\w+)*(?::\d+)?(?:,\w+(?:\.\w+)*(?::\d+)?)*\Z")
+def invalid_hosts(hosts):
+    """
+    vaguely matches a list of host1[:port1][,host2[:port2]],...
+    """
+    return _hosts.match(hosts) is None
